{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE RecordWildCards #-}

module Laskutin.Status where

import Data.Maybe (fromMaybe)
import Data.Text (Text)

import qualified Data.Text as T

import Laskutin.Types
import Laskutin.CSV

data InvoiceStatus = InvoiceStatus
    { nPaid :: Int
    , nTotal :: Int
    , amountPaid :: Euro
    , amountMissing :: Euro
    , amountTotal :: Euro
    } deriving Show

invoiceStatus :: [CsvInvoice] -> InvoiceStatus
invoiceStatus = InvoiceStatus
    <$> length . filter csvIsPaid
    <*> length
    <*> sum . fmap (\CsvInvoice {paid} -> fromMaybe 0 paid)
    <*> sum . fmap (\invoice@CsvInvoice {paid} -> max 0 $ csvInvoiceSum invoice - fromMaybe 0 paid)
    <*> sum . fmap csvInvoiceSum

showStatus :: InvoiceStatus -> Text
showStatus InvoiceStatus {..} =
    "Laskuja maksettu: " <> T.pack (show nPaid) <> "\n" <> 
    "Laskuja maksamatta: " <> T.pack (show $ nTotal - nPaid) <> "\n" <>
    "Laskuja yhteensä: " <> T.pack (show nTotal) <> "\n" <>
    "\n" <>
    "Maksettu: " <> renderEuro amountPaid <> "\n" <>
    "Maksamatta: " <> renderEuro amountMissing <> "\n" <>
    "Yhteensä saatavia: " <> renderEuro amountTotal <> "\n" <>
    "Ylimaksettu: " <> renderEuro (amountMissing + amountPaid - amountTotal) <> "\n"
